#include "smitten.hpp"
#include "SmittenUI.hpp"
#include "SmittenMenuHandler.hpp"
#include "skinfactory.hpp"
#include "menutypes.hpp"
#include "fsProcess.hpp"
#include "fsHandler.hpp"


// TODO: permit mouse pointer to select arbitrary item

SmittenUI *SmittenUI::m_pSingleton;

SmittenUI *SmittenUI::get() {
	if (!m_pSingleton) {
		m_pSingleton = new SmittenUI();
	}
	return m_pSingleton;
}

SmittenUI::SmittenUI() {
	changeMode(ModeBrowse);
	nonActionTime = 0;

	// two-stage construction for those platforms that don't allow throwing
	// of exceptions in ctor's.
	m_pMenu = new SmittenMenuHandler();
	m_pMedia = new MediaHandler();
	
	m_pSkin = SkinFactory::create(Config::get()->m_SkinName, Config::get()->m_VisualizerName);
		
	m_bIsDirty = true;
	m_pTextViewerData = NULL;
}

void SmittenUI::create() {
	m_pSkin->create();
	m_pMenu->rootMenu();
}

SmittenUI::~SmittenUI() {
	delete m_pMenu;
	delete m_pSkin;
	delete m_pMedia;
}

void SmittenUI::onAction(int action) {
	nonActionTime = 0;
	switch(m_Mode) {
		case ModeSlideshow:
			switch(action) {
				case ActionGoBack:
					exitSlideshow();
					break;
				case ActionGoForward:
				case ActionSelect:
					nextSlide();
					break;
				case ActionPrevious:
					break;
			}
			break;
		case ModeScreensaver:
			exitSlideshow();
			break;
		case ModeTextview:
			switch(action) {
				case ActionSelect:
					exitTextview();
					break;
				case ActionGoBack:
				case ActionSkipBackwards:
					if (m_iCurrentLine < m_pSkin->getTextPageSize()) {
						m_iCurrentLine = 0;
					} else {
						m_iCurrentLine -= m_pSkin->getTextPageSize();
					}
					m_bIsDirty = true;
					break;

				case ActionGoForward:
				case ActionSkipForward:
					m_iCurrentLine += m_pSkin->getTextPageSize();
					if (m_iCurrentLine >= m_pTextViewerData->getLineCount()) {
						m_iCurrentLine = m_pTextViewerData->getLineCount()-1;
					}
					m_bIsDirty = true;
					break;

				case ActionSkipStart:
					m_iCurrentLine = 0;
					m_bIsDirty = true;
					break;
				case ActionSkipEnd:
					m_iCurrentLine = m_pTextViewerData->getLineCount() - m_pSkin->getTextPageSize()/2;
					if (m_iCurrentLine < 0) m_iCurrentLine = 0;
					m_bIsDirty = true;
					break;

				case ActionPrevious:
					if (m_iCurrentLine != 0) {
						--m_iCurrentLine;
						m_bIsDirty = true;
					}
					break;
				case ActionNext:
					if (m_iCurrentLine < m_pTextViewerData->getLineCount()-1) {
						++m_iCurrentLine;
						m_bIsDirty = true;
					}
					break;
			}
			break;

		case ModeBrowse:
			switch(action) {

				case ActionSelect:
					m_pMenu->select();
					break;
				case ActionGoForward:
					m_pMenu->moveRight();
					break;
				case ActionGoBack:
					m_pMenu->moveLeft();
					break;
				case ActionNext:
					m_pMenu->moveDown();
					break;
				case ActionPrevious:
					m_pMenu->moveUp();
					break;
				case ActionRootMenu:
					m_pMenu->rootMenu();
					break;
				case ActionSkipStart:
					m_pMenu->moveStartList();
					m_bIsDirty = true;
					break;
				case ActionSkipEnd:
					m_pMenu->moveEndList();
					m_bIsDirty = true;
					break;
				case ActionSkipForward:
					m_pMenu->movePageDown(m_pSkin->getMenuPageSize());
					m_bIsDirty = true;
					break;
				case ActionSkipBackwards:
					m_pMenu->movePageUp(m_pSkin->getMenuPageSize());
					m_bIsDirty = true;
					break;
				case ActionQuit:
					changeMode(ModeExit);
					break;
			}
			break;
	}
}
	
bool SmittenUI::update(float t) {

	switch(m_Mode) {
		case ModeSlideshow:
		case ModeScreensaver:
			m_CurrentSlideDuration += t;
			if (m_CurrentSlideDuration > Config::get()->m_SlideshowTimeOut) {
				nextSlide();
			}
			break;
		case ModeExternalViewer:
			// mediaTime += t;
			// if time over, switch back to normal mode
			// WARNING: a crashed movie would hang the viewer
			break;
		case ModeExit:
			return false;
		default:
			{
				bool wasMoving = m_pMenu->isMoving();
				bool isMoving;

				m_pMenu->update(t);

				isMoving = m_pMenu->isMoving();

				if (wasMoving && !isMoving) {
					MenuItem *mi = m_pMenu->getCurrentMenuItem();
					MediaImageFileItem *pCurrent = dynamic_cast<MediaImageFileItem *>(mi);

					if (pCurrent && pCurrent->m_Preview == NULL && Config::get()->m_AutoPreviewImages) {
						SmittenImage *pImage = SmittenImage::create(pCurrent->m_Filepath);
						pCurrent->m_Preview = pImage->resize(m_pSkin->getPreviewImageWidth(), m_pSkin->getPreviewImageHeight(), true);
					}

				}
	
				nonActionTime += t;
				if (nonActionTime > Config::get()->m_ScreensaverTimeout) {
					startScreensaver();
				}

				m_bIsDirty |= m_pSkin->update(t);
				m_bIsDirty |= (wasMoving != isMoving);
				m_bIsDirty |= wasMoving;
			}
			break;
	}
	return true;
}

bool SmittenUI::draw() {
	if (!m_bIsDirty) {
		return false;
	}
	//
	switch(m_Mode) {
		case ModeSlideshow:
		case ModeScreensaver:
			{ 
				MediaImageFileItem *pCurrent = dynamic_cast<MediaImageFileItem *>(m_pSlideshowDirectory->m_List[m_CurrentSlide]);
				if (pCurrent && pCurrent->m_pFullimage) {
					int x = (m_pScreenImage->getWidth()- pCurrent->m_pFullimage->getWidth())/2;
					int y = (m_pScreenImage->getHeight() - pCurrent->m_pFullimage->getHeight())/2;
					m_pScreenImage->clear(0);
					pCurrent->m_pFullimage->draw(m_pScreenImage, x, y);
				}
			}
			break;
		case ModeTextview:
			m_pScreenImage->clip();
			m_pScreenImage->clear(0);
			m_pSkin->drawTextFile(m_pTextViewerData, m_iCurrentLine, m_iHorizontalOffset,0,0);
			break;

		case ModeExternalViewer:
			return false;
		default:
			m_pSkin->draw(SmittenUI::m_pMenu);
	}
	return true;
}

void SmittenUI::startTextview(MenuItem *pMenuItem) {
	MediaTextFileItem *pTextFile = dynamic_cast<MediaTextFileItem *>(pMenuItem);
	if (pTextFile) {
		m_pTextViewerData = new sgxTextFile(pTextFile->m_Filepath);
		if (m_pTextViewerData->getLineCount()) {
			changeMode(ModeTextview);
			m_iCurrentLine = 0;
			m_iHorizontalOffset = 0;
		}

	}
}

void SmittenUI::exitTextview() {
	changeMode(ModeBrowse);
	delete m_pTextViewerData;
}

	
void SmittenUI::changeMode(int newMode) {
	m_Mode = newMode;
	m_bIsDirty = true;
}

/*
**
*/
void SmittenUI::startScreensaver() {
	changeMode(ModeScreensaver);
	m_pSlideshowDirectory = new MenuList();

	std::vector<fsEntry *> directory;
	fsProcess::getDirectory(directory, Config::get()->m_ScreensaverDirectory);
	fsHandler::buildMenu(m_pSlideshowDirectory->m_List, directory);

	m_LoopSlideshow = true;

	// this wraps to 0 on 'next slide', despite being a unsigned/signed issue
	m_CurrentSlide = (unsigned int)-1;	
	//
	nextSlide();
}
	

/*
**
*/
void SmittenUI::startSlideshow(MenuList *pMenu, int start, bool loop) {
	changeMode(ModeSlideshow);

	m_pSlideshowDirectory = pMenu;
	m_LoopSlideshow = loop;

	m_CurrentSlide = start-1;
	nextSlide();
}

void SmittenUI::exitSlideshow() {
	if (m_Mode == ModeScreensaver) {
		delete m_pSlideshowDirectory;
		m_pSlideshowDirectory = NULL;
		nonActionTime = 0;
	}
	changeMode(ModeBrowse);
}

void SmittenUI::nextSlide() {
	MediaImageFileItem *mi;
	
	// Release the existing slide
	// (This isn't the cleanest way to do release the images), but we might not have
	// the memory to hold all slides in memory
	if (m_CurrentSlide < m_pSlideshowDirectory->m_List.size()) {
		mi = dynamic_cast<MediaImageFileItem *>(m_pSlideshowDirectory->m_List[m_CurrentSlide]);
		if (mi) {
			delete mi->m_pFullimage;
			mi->m_pFullimage = NULL;
		}
	}


	m_bIsDirty = true;

	do {
		if (++m_CurrentSlide >= m_pSlideshowDirectory->m_List.size()) {
			if (m_LoopSlideshow) {
				m_CurrentSlide = 0;
			} else {
				exitSlideshow();
				return;
			}
		}
		m_CurrentSlideDuration = 0;

		mi = dynamic_cast<MediaImageFileItem *>(m_pSlideshowDirectory->m_List[m_CurrentSlide]);
	} while(mi == NULL);

	SmittenImage *pImage = SmittenImage::create(mi->m_Filepath);
	if (pImage) {
		if (!mi->m_pFullimage) {
			mi->m_pFullimage = pImage->resize(m_pScreenImage->getWidth(), m_pScreenImage->getHeight(), true);
		}
		if (!mi->m_Preview) {
			mi->m_Preview = pImage->resize(m_pSkin->getPreviewImageWidth(), m_pSkin->getPreviewImageHeight(), true);
		}
	}
	delete pImage;
}

void SmittenUI::buildRootMenu(std::vector<MenuItem *> &menu) {

	std::vector<ConfigDir>::const_iterator it = Config::get()->m_DirectoryList.begin();

	for(;it!=Config::get()->m_DirectoryList.end();++it) {
		menu.push_back(new MediaDirectoryItem((*it).name, (*it).path));
	}

}

void SmittenUI::setScreenImage(SmittenImage *pScreen) {
	m_pScreenImage = pScreen;
}

SmittenImage *SmittenUI::getScreenImage() {
	return m_pScreenImage;
}
