#include "sdl/sdl.h"
#include "SDL/SDL_Image.h"
#include "SDL_stretch.h"
#include "smitten.hpp"

SmittenImage *SmittenImage::create(void *pSurface) {
	SmittenImage *pImg = new SmittenImage();
	pImg->m_pData = pSurface;
	pImg->m_bPublicSurface = true;
	return pImg;
}

SmittenImage *SmittenImage::create(const std::string &filename) {
	SmittenImage *pImg = new SmittenImage();
	pImg->m_pData = IMG_Load(filename.c_str());

	if (pImg->m_pData) {
		return pImg;
	}
	delete pImg;
	return NULL;
}

SmittenImage::SmittenImage() {
	m_pData = NULL;
	m_Locked = 0;
	m_bPublicSurface = false;
}

SmittenImage::~SmittenImage() {
	if (m_pData && !m_bPublicSurface) {
		SDL_FreeSurface((SDL_Surface *)m_pData);
	}
}

SmittenImage *SmittenImage::resize(int newWidth, int newHeight, bool keepAspect) {
	SDL_Surface *pSourceSurface = (SDL_Surface *)m_pData;
	SDL_Rect src = {0,0,(Uint16)pSourceSurface->w,(Uint16)pSourceSurface->h};
	SDL_Rect dest= {0,0,(Uint16)newWidth,(Uint16)newHeight};

	if (keepAspect) {
		// Method: look for longest side, and scale other axis to that
		if (pSourceSurface->w > pSourceSurface->h) {
			dest.h = (Uint16) ((dest.h * pSourceSurface->h) / pSourceSurface->w);
		} else {
			dest.w = (Uint16) ((dest.w * pSourceSurface->w) / pSourceSurface->h);
		}
	}
	
	SDL_Surface *pTargetSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, dest.w, dest.h, pSourceSurface->format->BitsPerPixel, pSourceSurface->format->Rmask, pSourceSurface->format->Gmask, pSourceSurface->format->Bmask, pSourceSurface->format->Amask);
	SDL_StretchSurfaceRect(pSourceSurface, &src, pTargetSurface, &dest);

	// Package up into SmittenImage
	SmittenImage *pImg = new SmittenImage();
	pImg->m_pData = pTargetSurface;

	return pImg;
}

int SmittenImage::getWidth() {
	return ((SDL_Surface*)m_pData)->w;
}

int SmittenImage::getHeight() {
	return ((SDL_Surface*)m_pData)->h;
}

void SmittenImage::draw(SmittenImage *pImageSurface, int x, int y) {
	SDL_Surface *pSurface = (SDL_Surface *)pImageSurface->m_pData;
	SDL_Rect dest = {(Uint16)x,(Uint16)y,0,0};
	SDL_BlitSurface((SDL_Surface *)m_pData, NULL, pSurface, &dest);
}

void SmittenImage::drawArea(SmittenImage *pImageSurface, int x, int y, int sx, int sy, int sx2, int sy2) {
	SDL_Surface *pSurface = (SDL_Surface *)pImageSurface->m_pData;
	SDL_Rect src = {(Uint16)sx, (Uint16)sy, (Uint16)(sx2-sx), (Uint16)(sy2-sy)};
	SDL_Rect dest = {(Uint16)x, (Uint16)y,0,0};
	SDL_BlitSurface((SDL_Surface *)m_pData, &src, pSurface, &dest);
}

void SmittenImage::clear(unsigned int color) {
	SDL_FillRect((SDL_Surface *)m_pData, NULL, color);
}

void SmittenImage::clip() {
	SDL_SetClipRect((SDL_Surface *)m_pData, NULL);
}

void SmittenImage::clip(const sRect &rc) {
	SDL_Rect r = { (Uint16)rc.x, (Uint16)rc.y, (Uint16)rc.w, (Uint16)rc.h};
	SDL_SetClipRect((SDL_Surface *)m_pData, &r);
}

void SmittenImage::clip(int x, int y, int w, int h) {
	SDL_Rect r = { (Sint16)x, (Sint16)y, (Uint16)w, (Uint16)h};
	SDL_SetClipRect((SDL_Surface *)m_pData, &r);
}

void SmittenImage::fillRect(const sRect &rect, int r, int g, int b) {
	unsigned int col = SDL_MapRGB(((SDL_Surface *)m_pData)->format, (Uint8)r,(Uint8)g,(Uint8)b);
	SDL_Rect rc = { (Sint16)rect.x, (Sint16)rect.y, (Uint16)rect.w, (Uint16)rect.h};
	SDL_FillRect((SDL_Surface *)m_pData, &rc, col);
}

SmittenImage *SmittenImage::cloneArea(int x, int y, int x2, int y2) {
	SDL_Surface *pSurface = (SDL_Surface *)m_pData;
	
	if (x<0) x = 0;
	if (y<0) y = 0;
	if (x2>pSurface->w) x2 = pSurface->w;
	if (y2>pSurface->h) y2 = pSurface->h;

	SDL_Surface *pTargetSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, x2-x,y2-y, pSurface->format->BitsPerPixel, pSurface->format->Rmask, pSurface->format->Gmask, pSurface->format->Bmask, pSurface->format->Amask);
	
	SDL_Rect dest = {(Sint16)x,(Sint16)y, (Uint16)(x2-x), (Uint16)(y2-y)};
	SDL_BlitSurface(pSurface, &dest, pTargetSurface, NULL);

	SmittenImage *pImg = new SmittenImage();
	pImg->m_pData = pTargetSurface;
	return pImg;
}

void SmittenImage::setTransparency(int col) {
	SDL_SetColorKey((SDL_Surface *)m_pData, SDL_SRCCOLORKEY, col);
}

unsigned int SmittenImage::lockPixels() {
	SDL_LockSurface((SDL_Surface *)m_pData);
	++m_Locked;
	return m_Locked;
}

unsigned int SmittenImage::unlockPixels() {
	SDL_UnlockSurface((SDL_Surface *)m_pData);
	--m_Locked;
	return m_Locked;
}

unsigned int SmittenImage::getColor(int x, int y) {
	SDL_Surface *pSurface = (SDL_Surface *)m_pData;
    int bpp = pSurface->format->BytesPerPixel;
	int locked = m_Locked;

	if (!locked) {
		SDL_LockSurface(pSurface);
	}

    Uint8 *p = (Uint8 *)pSurface->pixels + y * pSurface->pitch + x * bpp;

    unsigned int color = 0;

    switch(bpp) {
      case 1:
          color = *p;
          break;

      case 2:
          color = *(Uint16 *)p;
          break;

      case 3:
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
              color = p[0] << 16 | p[1] << 8 | p[2];
#else
              color = p[0] | p[1] << 8 | p[2] << 16;
#endif
          break;

      case 4:
          color = *(unsigned int *)p;
          break;

    }

	if (!locked) {
		SDL_UnlockSurface(pSurface);
	}
	return color;
}



